###############
# 解説1
###############
import ctypes
import tisgrabber as tis
import numpy as np
import cv2

ic = ctypes.cdll.LoadLibrary("./tisgrabber_x64.dll")
tis.declareFunctions(ic)

###############
# 解説2
###############
class CallbackUserdata(ctypes.Structure):
    def __init__(self):
        self.Value1 = 0
        self.Value2 = 0
        self.BufferSize=200
        self.end = False
        self.firstindex = -1
        
###############
# 解説3
###############
def FrameCallback(hGrabber, hMemBuffer, framenumber, pData):
    pData.Value1 = pData.Value1 + 1
    # リングバッファの最初の画像のインデックスを取得します
    if( pData.firstindex == -1 ):
        Index = ctypes.c_long()
        ic.IC_MemBufferGetIndex(hMemBuffer, Index)
        pData.firstindex = Index.value

###############
# 解説4
###############
def saveimage(hGrabber, i):
#### 解説4-1

    membuffer = ctypes.c_void_p()
    ic.IC_GetMemBuffer(hGrabber,i, membuffer)

#### 解説4-2
    Width = ctypes.c_long()
    Height = ctypes.c_long()
    BitsPerPixel = ctypes.c_int()
    ic.IC_GetMemBufferDescription( membuffer,Width, Height, BitsPerPixel)
    bpp = int(BitsPerPixel.value / 8.0)
    buffer_size = Width.value * Height.value * BitsPerPixel.value

    # イメージデータを取得
    imagePtr = ctypes.c_char_p()
    ic.IC_MemBufferGetDataPtr(membuffer, imagePtr)

    imagedata = ctypes.cast(imagePtr,
                            ctypes.POINTER(ctypes.c_ubyte *
                                            buffer_size))
#### 解説4-3
    # numpy配列を作成します
    image = np.ndarray(buffer=imagedata.contents,
                        dtype=np.uint8,
                        shape=(Height.value,
                               Width.value,
                               bpp))
#### 解説4-4
    image = cv2.flip(image, 0) #画像を反転させない
    cv2.imwrite("test{0}.png".format(i),image) 

    # 空きメモリ、もう必要ありません。
    # 画像はまだリングバッファにあります。
    ic.IC_ReleaseMemBuffer( membuffer )
    

###############
# 解説5
###############
#この関数は、このライブラリの他の関数が呼び出される前に1回だけ呼び出す必要があります。
ic.IC_InitLibrary(0)
#ユーザーデータをインスタンス化
Userdata = CallbackUserdata()
# 関数ポインタを作成
Callbackfuncptr = ic.FRAMEREADYCALLBACKEX(FrameCallback)
#ダイアログ画面を表示
hGrabber = tis.openDevice(ic)


###############
# 解説6
###############
#### 解説6-1
#デバイスが有効か確認
if(ic.IC_IsDevValid(hGrabber)):

    ic.IC_SetFrameReadyCallbackEx(hGrabber, Callbackfuncptr, Userdata)
    
    #リングバッファの枚数
    ic.IC_SetRingBufferSize(hGrabber,Userdata.BufferSize)
    ic.IC_SetContinuousMode(hGrabber, 0) 
    ic.IC_StartLive(hGrabber, 0) #ライブスタート開始　引数：0の時非表示、引数：1の時表示
    
#### 解説6-2
    #リングバッファに200（Userdata.BufferSize）枚撮りためるまで待つ
    while (Userdata.Value1<Userdata.BufferSize):
        cv2.waitKey(1)    
    ic.IC_StopLive(hGrabber)

#### 解説6-3
    # 取得した画像を保存する
    for i in range(Userdata.firstindex, Userdata.Value1+Userdata.firstindex-1):
        saveimage(hGrabber, i)

else:
    ic.IC_MsgBox(tis.T("No device opened"), tis.T("Callback"))

ic.IC_ReleaseGrabber(hGrabber)



